#!/usr/bin/env python

import json
from os import getcwd
from os.path import join
from .aws_controller import AWSController
from cement.core.controller import CementBaseController, expose
from cement.core.foundation import CementApp
from .core import Core


class MyBaseController(CementBaseController):
    class Meta:
        label = 'base'
        description = 'Basic Commands for aws_site_maker'
        arguments = [
            (['-l', '--location'], dict(action='store',
                                        help='specify the directory to run on', metavar='STR')),
            (['-c', '--config'], dict(action='store',
                                      help='config file name', metavar='STR')),
            (['extra_arguments'], dict(action='store', nargs='*'))
        ]

    def load_config_file(self, config_file_path):
        with open(config_file_path) as f:
            return json.load(f)

    def get_config_file_path(self):
        if self.app.pargs.location:
            location = self.app.pargs.location
        else:
            location = getcwd()

        if self.app.pargs.config:
            config_file_path = join(location, self.app.pargs.config)
        else:
            config_file_path = join(location, 'aws_site_maker.json')

        return config_file_path

    @expose(help='Initiate the aws_site_maker settings')
    def init(self):
        core = Core()
        core.init()        

    @expose(help="deploy to s3")
    def deploy(self):
        if self.app.pargs.extra_arguments[0]:
            environment = self.app.pargs.extra_arguments[0]
        else:
            print('Please specify an environment')
            return

        config_file = self.load_config_file(self.get_config_file_path())

        if self.app.pargs.location:
            location = self.app.pargs.location
        else:
            location = getcwd()

        aws = AWSController()
        aws.deploy(location, config_file[environment])

    @expose(help='create an s3 bucket', hide=False)
    def create_bucket(self):
        if self.app.pargs.extra_arguments[0]:
            bucket_name = self.app.pargs.extra_arguments[0]
        else:
            print('No bucket name was specified')
            return

        aws = AWSController()
        aws.create_s3_bucket(bucket_name, 'private', {
                             'LocationConstraint': 'us-west-2'})

    @expose(help='undeploy the environment')
    def undeploy(self):
        if self.app.pargs.extra_arguments[0]:
            environment = self.app.pargs.extra_arguments[0]
        else:
            print('Please specify an environment')
            return

        config_file = self.load_config_file(self.get_config_file_path())
        environment = config_file[environment]
        aws = AWSController()
        aws.undeploy(environment)


class MyApp(CementApp):
    class Meta:
        label = 'myapp'
        base_controller = 'base'
        handlers = [MyBaseController]
def handler():
    with MyApp() as app:
        app.run()



from os import listdir
from os.path import isfile, join, isdir, basename, splitext
import sys
import boto3
from botocore.client import ClientError


class AWSController:
    def __init__(self):
        self.s3_client = boto3.resource('s3')

    def put_directory_in_bucket(self, bucket_name, bucket_folder, dir_location, is_recursive=True, ignore_list=None):
        dir_list = [f for f in listdir(
            dir_location) if f not in ignore_list]

        if ignore_list:
            dir_list = [f for f in dir_list if f not in ignore_list]

        for fn in dir_list:
            if isfile(join(dir_location, fn)):
                key = fn

                if bucket_folder:
                    key = join(bucket_folder, fn)

                self.upload_file_to_s3_bucket(bucket_name, join(dir_location, fn), key)

            elif isdir(join(dir_location, fn)) and is_recursive:
                if bucket_folder:
                    fn = join(bucket_folder, fn)

                print("Creating folder %s" % fn)
                self.create_bucket_folder(bucket_name, fn)

                self.put_directory_in_bucket(
                    bucket_name,
                    fn,
                    join(dir_location, basename(fn)),
                    is_recursive,
                    ignore_list)


    def create_bucket_folder(self, bucket_name, folder_name):
        try:
            response = self.s3_client.Bucket(bucket_name).put_object(ACL='public-read', Body='', Key=folder_name + '/')
        except ClientError:
            print('Was unable to create the folder')
        return response

    def deploy(self, dir_location, environment):
        bucket = environment.get('s3_bucket')
        ignore_list = environment.get('ignore')

        # response = self.delete_objects_in_bucket(bucket_name)
        try:
            self.s3_client.meta.client.head_bucket(Bucket=bucket)
        except ClientError:
            print("Creating the bucket {0}...".format(bucket))

            region = boto3.session.Session().region_name
            self.s3_client.create_bucket(ACL='public-read',
                                         Bucket=bucket,
                                         CreateBucketConfiguration={'LocationConstraint': region})


        self.put_directory_in_bucket(bucket, None, dir_location, True, ignore_list)
        #TODO: make it optional to change the index and error files
        self.make_bucket_website(bucket, 'index.html', 'error.html')

    def get_file_content_type(self, filename_ext):
        mimetypes = {
            'html': 'text/html',
            'json': 'application/json',
            'css': 'text/css',
            'other': 'text/plain'
        }

        return mimetypes.get(filename_ext, mimetypes.get('other'))

    # TODO: Needs to be updated
    def get_bucket_objects(self, bucket_name):
        response = self.s3_client.meta.client.list_objects(
            Bucket=bucket_name
        )
        contents = response['Contents']
        formatted_contents = [{'Key': d['Key'], 'Size': d['Size'],
                               'LastModified': d['LastModified'].strftime("%B %d, %Y")} for d in contents]
        return formatted_contents

    def check_if_bucket_exists(self, bucket_name):
        try:
            boto3.client('s3').head_bucket(Bucket=bucket_name)
        except ClientError as e:
            print(e)
            return False

        return True

    def upload_file_to_s3_bucket(self, bucket_name, file_path, key):
        if not self.check_if_bucket_exists(bucket_name):
            print('Creating the {0} bucket...'.format(bucket_name)) 
        
        bucket = self.s3_client.Bucket(bucket_name)
        filename, file_extension = splitext(file_path)

        print('Uploading file %s...' % key)

        bucket.upload_file(file_path, key, ExtraArgs={
            'ACL': 'public-read', 'ContentType': self.get_file_content_type(file_extension[1:])})



    def undeploy(self, environment):
        bucket = environment.get('s3_bucket')
        response = self.delete_objects_in_bucket(bucket)

    def delete_objects_in_bucket(self, bucket_name):
        print('Deleting the %s bucket...' % bucket_name)
        bucket = self.s3_client.Bucket(bucket_name)

        object_list = self.get_bucket_objects(bucket_name)
        object_list = {'Objects': [{'Key': d['Key']} for d in object_list]}

        response = bucket.delete_objects(Delete=object_list)

        return response

    def make_bucket_website(self, bucket_name, index_file, error_file):
        bucket_website = self.s3_client.BucketWebsite(bucket_name)
        print('Setting the bucket %s as a website' % bucket_name)
        
        response = bucket_website.put(
            WebsiteConfiguration={
                'ErrorDocument': {
                    'Key': error_file
                },
                'IndexDocument': {
                    'Suffix': index_file
                }
            }
        )

        region = boto3.client('s3').get_bucket_location(Bucket=bucket_name)['LocationConstraint']
        url = '{0}.s3-website-{1}.amazonaws.com'.format(bucket_name, region)
        print('The website address is {0}'.format(url))



import json
import sys
import os

CONFIG_FILE_TEMPLATE = """{
    "dev": {
        "s3_bucket" : "*"
    }
}"""

AVAILABLE_REGIONS = {
    'US East (Ohio)' :	'us-east-2',
    'US East (N. Virginia)' : 'us-east-1',
    'US West (N. California)' :	'us-west-1',
    'US West (Oregon)' : 'us-west-2',
    'Canada (Central)': 'ca-central-1',
    'Asia Pacific (Mumbai)': 'ap-south-1',
    'Asia Pacific (Seoul)': 'ap-northeast-2',
    'Asia Pacific (Osaka-Local)': 'ap-northeast-3',
    'Asia Pacific (Singapore)': 'ap-southeast-1',
    'Asia Pacific (Sydney)': 'ap-southeast-2',
    'Asia Pacific (Tokyo)': 'ap-northeast-1',
    'China (Beijing)': 'cn-north-1',
    'China (Ningxia)': 'cn-northwest-1',
    'EU (Frankfurt)': 'eu-central-1',
    'EU (Ireland)': 'eu-west-1',
    'EU (London)': 'eu-west-2',
    'EU (Paris)': 'eu-west-3',
    'South America (Sao Paulo)': 'sa-east-1'
}

class Core:
    config_file_template = CONFIG_FILE_TEMPLATE

    #Creates the aws_site_maker.json file
    def init(self):
        bucket_name = input('Enter the bucket name: ')
        # region = input('Enter the region you want to create the bucket in: ')

        config_file_template_obj = json.loads(self.config_file_template)
        config_file_template_obj['dev']['s3_bucket'] = bucket_name
        # config_file_template_obj['Environments']['dev']['Region'] = region

        with open('aws_site_maker.json', 'w') as f:
            json.dump(config_file_template_obj, f)


    def load_config_file(self, config_file_path='aws_site_maker.json'):
        if os.path.exists(config_file_path):
            with open(config_file_path) as f:
                try:
                    self.config_file = json.load(f)
                except:
                    print('Was not able to load the json file')
        else:
            print('The config file at {0} does not exist or was not able to be read.'.format(config_file_path))


    ###############CONFIG FILE METHODS#####################
    def get_environment(self, environment_name):
        try:
            return self.config_file['Environments'][environment_name]
        except KeyError:
            print('The environment {0} was not found in the config file...'.format(environment_name))
            sys.exit(1)
        # response = self.delete_objects_in_bucket(bucket_name)

    def get_bucket_name(self):
        BUCKET_CONST = 's3_bucket'

        try:
            return self.environment[BUCKET_CONST]
        except:
            print('Could not find a bucket name in the config file...')
            sys.exit(1)



from setuptools import setup
import setuptools
from s3_site_maker import __version__
import os
import sys

try:
    from pypandoc import convert
    README = convert('README.md', 'rst')
except ImportError:
    README = open(os.path.join(os.path.dirname(__file__), 'README.md'), 'r', encoding="utf-8").read()

with open(os.path.join(os.path.dirname(__file__), 'requirements.txt')) as f:
    if sys.version_info[0] == 2:
        required = f.read().splitlines()
    else:
        required = []
        for package in f.read().splitlines():
            if 'futures' not in package:
                required.append(package)


setup(
    name='s3_site_maker',
    version=__version__,
    packages=['s3_site_maker'],
    install_requires=required,
    include_package_data=True,
    license='MIT License',
    description='Create a static webstie in an S3 Bucket',
    long_description=README,
    author='Alberto Acuna',
    author_email='aacuna3@asu.edu',
    entry_points={
        'console_scripts': [
            's3_site_maker=s3_site_maker.cli:handler'
        ]
    },
    classifiers=[
        'Environment :: Console',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.6',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)
